#!/usr/bin/env node
"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const graphql_1 = require("graphql");
const utils_1 = require("@graphql-tools/utils");
const url_loader_1 = require("@graphql-tools/url-loader");
const json_file_loader_1 = require("@graphql-tools/json-file-loader");
const graphql_file_loader_1 = require("@graphql-tools/graphql-file-loader");
const load_1 = require("@graphql-tools/load");
const fs = __importStar(require("fs"));
const yargs = __importStar(require("yargs"));
const path_1 = require("path");
// yargs command options config
const { path, url, file } = yargs.options({
    path: { type: 'string', alias: 'p', demandOption: true },
    url: { type: 'string' },
    file: { type: 'string' }
}).check((argv, options) => {
    if (!argv.file && !argv.url) {
        throw new Error('You must pass either an url or a file!');
    }
    else {
        return true;
    }
}).argv;
(() => __awaiter(void 0, void 0, void 0, function* () {
    var _a, _b, _c;
    // load schema
    const schema = yield load_1.loadSchema(url || file || '', {
        loaders: [
            new url_loader_1.UrlLoader(),
            new json_file_loader_1.JsonFileLoader(),
            new graphql_file_loader_1.GraphQLFileLoader()
        ]
    });
    // clean dir
    cleanDir(path_1.resolve(path));
    // get operations fields
    const mutations = (_a = schema.getMutationType()) === null || _a === void 0 ? void 0 : _a.getFields();
    const queries = (_b = schema.getQueryType()) === null || _b === void 0 ? void 0 : _b.getFields();
    const subscriptions = (_c = schema.getSubscriptionType()) === null || _c === void 0 ? void 0 : _c.getFields();
    // generate files
    const promisses = [];
    mutations && promisses.push(generateFilesForFields(schema, mutations, 'mutation'));
    queries && promisses.push(generateFilesForFields(schema, queries, 'query'));
    subscriptions && promisses.push(generateFilesForFields(schema, subscriptions, 'subscription'));
    yield Promise.all(promisses);
}))();
function generateFilesForFields(schema, obj, kind) {
    return __awaiter(this, void 0, void 0, function* () {
        const promisses = [];
        const opsPath = `${path_1.resolve(path)}/${kind === 'query' ? 'queries' : kind === 'mutation' ? 'mutations' : 'subscriptions'}`;
        // create operations directory if it doesnt exit
        if (!fs.existsSync(opsPath)) {
            fs.mkdirSync(opsPath);
        }
        // loop through each operation and create graphql file
        Object.keys(obj).forEach(field => {
            promisses.push(new Promise(resolve => {
                const ops = utils_1.buildOperationNodeForField({ schema, kind, field });
                const file = `${opsPath}/${field}.graphql`;
                const content = `${graphql_1.print(ops).replace(/^([\t\s]*)((?!\().)*(:\s)/gm, '$1').replace(/(\$\w)/g, (s) => s.toLowerCase())}`;
                fs.writeFile(file, content, () => resolve({ field: field.replace(/\.?([A-Z]+)/g, (x, y) => '_' + y).replace(/^_/, '').toUpperCase(), file }));
            }));
        });
        // create import file for operations
        const files = yield Promise.all(promisses);
        const lines = files.map(({ field, file }) => `export const ${field} = loader('${file.replace(opsPath, '.')}')`).join(`\r\n`);
        fs.writeFileSync(`${opsPath}/index.ts`, `import { loader } from 'graphql.macro'\r\n${lines}`);
    });
}
function cleanDir(dirPath, removeSelf = false) {
    if (fs.existsSync(dirPath)) {
        try {
            var files = fs.readdirSync(dirPath);
        }
        catch (e) {
            return;
        }
        if (files.length > 0)
            for (var i = 0; i < files.length; i++) {
                var filePath = dirPath + '/' + files[i];
                if (fs.statSync(filePath).isFile())
                    fs.unlinkSync(filePath);
                else
                    cleanDir(filePath, true);
            }
        removeSelf && fs.rmdirSync(dirPath);
    }
    else {
        fs.mkdirSync(dirPath);
    }
}
//# sourceMappingURL=index.js.map